package at.jku.ssw.pi.graph.ue07;

import at.jku.ssw.pi.Out;
import at.jku.ssw.pi.dot.DotMaker;
import at.jku.ssw.pi.graph.Edge;
import at.jku.ssw.pi.graph.Graph;
import at.jku.ssw.pi.graph.Vertex;

class GraphTest {

	public static void main(String[] args) {
		test1();
		test2();
		test3();
		test4();
		test5();
		test6();
		test7();
		test8();
		test9();
		test10();
		test11();
		test12();
	}

	static void makeDot(String filename, Graph graph) {
		Out.open(filename);
		Out.println(DotMaker.makeDotForGraph(graph));
		Out.close();
	}

	static void test1() {
		Out.println("\n*** Test 1 - make dot for graph ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, d));
		g.addEdge(new Edge(d, a));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, e));
		g.addEdge(new Edge(e, b));
		g.addEdge(new Edge(e, f));
		makeDot("1.dot", g);
	}

	static void test2() {
		Out.println("\n*** Test 2 - dfs, bfs ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, d));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, f));
		g.addEdge(new Edge(f, e));
		g.addEdge(new Edge(e, c));
		g.addEdge(new Edge(d, b));
		g.addEdge(new Edge(d, e));
		g.addEdge(new Edge(d, f));
		makeDot("2.dot", g);
		Out.println("\nDFS");
		Graphs.depthFirstSearch(g);
		Out.println("\nBFS");
		Graphs.breadthFirstSearch(g);
	}

	static void test3() {
		Out.println("\n*** Test 3 - dfs, bfs ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, d));
		g.addEdge(new Edge(a, e));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, e));
		g.addEdge(new Edge(e, b));
		g.addEdge(new Edge(e, f));
		makeDot("3.dot", g);
		Out.println("\nDFS");
		Graphs.depthFirstSearch(g);
		Out.println("\nBFS");
		Graphs.breadthFirstSearch(g);
	}

	static void test4() {
		Out.println("\n*** Test 4 - dfs, bfs ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, d));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, f));
		g.addEdge(new Edge(d, b));
		g.addEdge(new Edge(d, e));
		g.addEdge(new Edge(d, f));
		g.addEdge(new Edge(e, c));
		g.addEdge(new Edge(f, e));
		makeDot("4.dot", g);
		Out.println("\nDFS");
		Graphs.depthFirstSearch(g);
		Out.println("\nBFS");
		Graphs.breadthFirstSearch(g);
	}

	static void test5() {
		Out.println("\n*** Test 5 - dfs, bfs, mst, sp (undirected, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		g.addEdge(new Edge(a, b, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(b, c, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(b, d, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(c, a, 0, Edge.Kind.Undirected));
		Out.println("\nDFS");
		Graphs.depthFirstSearch(g);
		Out.println("\nBFS");
		Graphs.breadthFirstSearch(g);
		makeDot("5a.dot", g);
		Out.println("\nMST");
		Graph mst = Graphs.makeMinimalSpanningTree(g, a);
		makeDot("5b.dot", mst);
		Out.println("\nSP");
		Graph sp = Graphs.makeShortestPath(g, a, d);
		makeDot("5c.dot", sp);
	}

	static void test6() {
		Out.println("\n*** Test 6 - mst (undirected, weighted) ***");
		Graph graph = new Graph();
		Vertex a = new Vertex('A');
		graph.addVertex(a);
		Vertex b = new Vertex('B', true);
		graph.addVertex(b);
		Vertex c = new Vertex('C');
		graph.addVertex(c);
		Vertex d = new Vertex('D');
		graph.addVertex(d);
		Vertex e = new Vertex('E');
		graph.addVertex(e);
		Vertex f = new Vertex('F');
		graph.addVertex(f);
		Vertex g = new Vertex('G');
		graph.addVertex(g);
		graph.addEdge(new Edge(a, g, 6, Edge.Kind.Undirected));
		graph.addEdge(new Edge(a, b, 1, Edge.Kind.Undirected));
		graph.addEdge(new Edge(a, f, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(b, c, 1, Edge.Kind.Undirected));
		graph.addEdge(new Edge(b, d, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(b, e, 4, Edge.Kind.Undirected));
		graph.addEdge(new Edge(c, e, 4, Edge.Kind.Undirected));
		graph.addEdge(new Edge(d, f, 1, Edge.Kind.Undirected));
		graph.addEdge(new Edge(d, e, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(e, f, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(e, g, 1, Edge.Kind.Undirected));
		makeDot("6a.dot", graph);
		Out.println("\nMST");
		Graph mst = Graphs.makeMinimalSpanningTree(graph, b);
		makeDot("6b.dot", mst);
	}

	static void test7() {
		Out.println("\n*** Test 7 - sp (undirected, weighted) ***");
		Graph graph = new Graph();
		Vertex a = new Vertex('A', true);
		graph.addVertex(a);
		Vertex b = new Vertex('B');
		graph.addVertex(b);
		Vertex c = new Vertex('C');
		graph.addVertex(c);
		Vertex d = new Vertex('D');
		graph.addVertex(d);
		Vertex e = new Vertex('E');
		graph.addVertex(e);
		Vertex f = new Vertex('F');
		graph.addVertex(f);
		Vertex g = new Vertex('G');
		graph.addVertex(g);
		graph.addEdge(new Edge(a, g, 6, Edge.Kind.Undirected));
		graph.addEdge(new Edge(a, b, 1, Edge.Kind.Undirected));
		graph.addEdge(new Edge(a, f, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(b, c, 1, Edge.Kind.Undirected));
		graph.addEdge(new Edge(b, d, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(b, e, 4, Edge.Kind.Undirected));
		graph.addEdge(new Edge(c, e, 4, Edge.Kind.Undirected));
		graph.addEdge(new Edge(d, f, 1, Edge.Kind.Undirected));
		graph.addEdge(new Edge(d, e, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(e, f, 2, Edge.Kind.Undirected));
		graph.addEdge(new Edge(e, g, 1, Edge.Kind.Undirected));
		makeDot("7a.dot", graph);
		Out.println("\nSP");
		Graph mst = Graphs.makeShortestPath(graph, a, g);
		makeDot("7b.dot", mst);
	}

	static void test8() {
		Out.println("\n*** Test 8 - dfs, bfs (directed, weighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b, 1, Edge.Kind.Directed));
		g.addEdge(new Edge(a, d, 2, Edge.Kind.Directed));
		g.addEdge(new Edge(b, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, f, Edge.Kind.Directed));
		g.addEdge(new Edge(d, b, 1, Edge.Kind.Directed));
		g.addEdge(new Edge(d, e, 2, Edge.Kind.Directed));
		g.addEdge(new Edge(d, f, 3, Edge.Kind.Directed));
		g.addEdge(new Edge(e, c, Edge.Kind.Directed));
		Out.println("\nDFS");
		Graphs.depthFirstSearch(g);
		Out.println("\nBFS");
		Graphs.breadthFirstSearch(g);
		makeDot("8.dot", g);
	}

	static void test9() {
		Out.println("\n*** Test 9 - sp (undirected, weighted) ***");
		Graph g = new Graph();
		Vertex v1 = new Vertex('A', true);
		g.addVertex(v1);
		Vertex v2 = new Vertex('B');
		g.addVertex(v2);
		Vertex v3 = new Vertex('C');
		g.addVertex(v3);
		Vertex v4 = new Vertex('D');
		g.addVertex(v4);
		Vertex v5 = new Vertex('E');
		g.addVertex(v5);
		Vertex v6 = new Vertex('F');
		g.addVertex(v6);
		Vertex v7 = new Vertex('G');
		g.addVertex(v7);
		Vertex v8 = new Vertex('H');
		g.addVertex(v8);
		Vertex v9 = new Vertex('I');
		g.addVertex(v9);
		Vertex v10 = new Vertex('J');
		g.addVertex(v10);
		Vertex v11 = new Vertex('K');
		g.addVertex(v11);
		Vertex v12 = new Vertex('L');
		g.addVertex(v12);
		Vertex v13 = new Vertex('M');
		g.addVertex(v13);
		Vertex v14 = new Vertex('N');
		g.addVertex(v14);
		Vertex v15 = new Vertex('O');
		g.addVertex(v15);
		g.addEdge(new Edge(v1, v2, 18, Edge.Kind.Undirected));
		g.addEdge(new Edge(v2, v3, 15, Edge.Kind.Undirected));
		g.addEdge(new Edge(v3, v4, 15, Edge.Kind.Undirected));
		g.addEdge(new Edge(v1, v4, 12, Edge.Kind.Undirected));
		g.addEdge(new Edge(v1, v15, 15, Edge.Kind.Undirected));
		g.addEdge(new Edge(v1, v5, 20, Edge.Kind.Undirected));
		g.addEdge(new Edge(v4, v6, 20, Edge.Kind.Undirected));
		g.addEdge(new Edge(v5, v6, 20, Edge.Kind.Undirected));
		g.addEdge(new Edge(v14, v15, 15, Edge.Kind.Undirected));
		g.addEdge(new Edge(v14, v5, 25, Edge.Kind.Undirected));
		g.addEdge(new Edge(v5, v7, 18, Edge.Kind.Undirected));
		g.addEdge(new Edge(v6, v8, 14, Edge.Kind.Undirected));
		g.addEdge(new Edge(v7, v8, 15, Edge.Kind.Undirected));
		g.addEdge(new Edge(v14, v13, 50, Edge.Kind.Undirected));
		g.addEdge(new Edge(v13, v11, 10, Edge.Kind.Undirected));
		g.addEdge(new Edge(v11, v10, 35, Edge.Kind.Undirected));
		g.addEdge(new Edge(v13, v12, 70, Edge.Kind.Undirected));
		g.addEdge(new Edge(v12, v9, 60, Edge.Kind.Undirected));
		g.addEdge(new Edge(v9, v10, 15, Edge.Kind.Undirected));
		g.addEdge(new Edge(v7, v10, 10, Edge.Kind.Undirected));
		g.addEdge(new Edge(v8, v9, 20, Edge.Kind.Undirected));
		makeDot("9a.dot", g);
		Out.println("\nSP");
		Graph sp = Graphs.makeShortestPath(g, v4, v10);
		makeDot("9b.dot", sp);
	}

	static void test10() {
		Out.println("\n*** Test 10 - dfs, bfs (directed, weighted) ***");
		// Bespiel im Vortrag 08 (SS2011)
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(b, d, 1, Edge.Kind.Directed));
		g.addEdge(new Edge(b, e, 2, Edge.Kind.Directed));
		g.addEdge(new Edge(d, c, Edge.Kind.Directed));
		g.addEdge(new Edge(e, a, Edge.Kind.Directed));
		makeDot("10.dot", g);
		Graphs.depthFirstSearch(g);
		Graphs.breadthFirstSearch(g);
	}

	static void test11() {
		Out.println("\n*** Test 11 - makedot (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('1', true);
		g.addVertex(a);
		Vertex b = new Vertex('2');
		g.addVertex(b);
		Vertex c = new Vertex('3');
		g.addVertex(c);
		Vertex d = new Vertex('4');
		g.addVertex(d);
		Vertex e = new Vertex('5');
		g.addVertex(e);
		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(b, c, Edge.Kind.Directed));
		g.addEdge(new Edge(b, e, Edge.Kind.Directed));
		g.addEdge(new Edge(a, d, Edge.Kind.Directed));
		g.addEdge(new Edge(d, a, Edge.Kind.Directed));
		makeDot("11.dot", g);
	}

	static void test12() {
		Out.println("\n*** Test 12 - dfs, bfs (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(a, d, Edge.Kind.Directed));
		g.addEdge(new Edge(a, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, f, Edge.Kind.Directed));
		g.addEdge(new Edge(d, b, Edge.Kind.Directed));
		g.addEdge(new Edge(d, e, Edge.Kind.Directed));
		g.addEdge(new Edge(d, f, Edge.Kind.Directed));
		g.addEdge(new Edge(e, c, Edge.Kind.Directed));
		g.addEdge(new Edge(f, e, Edge.Kind.Directed));
		makeDot("12.dot", g);
		Out.println("\nDFS");
		Graphs.depthFirstSearch(g);
		Out.println("\nBFS");
		Graphs.breadthFirstSearch(g);
	}
}